// J-INTERCAL 0.12 Copyright (c) 2000-2001 Karsten Sperling, Karlsruhe, Germany.
// This software is released under the GNU LGPL, see LICENSE for details.

package intercal;

public class IntercalException extends Exception {
  protected int line, error;
  
  public IntercalException(String msg, int line) {
    super(msg);
    this.line  = line;
    this.error = 0;
  }
  
  public IntercalException(int error, int line) {
    super(getErrorMessage(error));
    this.line  = line;
    this.error = error;
  }

  public IntercalException(String msg) {
    this(msg, -1);
  }
  
  public IntercalException(int error) {
    this(error, -1);
  }
  
  public int getLine() { return line; }
  
  public void setLine(int line) { if (this.line == -1) this.line = line; }
  
  public int getError() { return error; }
  
  public String toString() {
    String msg;
    
    msg = String.valueOf(error);
    while (msg.length() < 3) msg = "0" + msg;
    
    msg = "ICL" + msg + "I\t" + getMessage();
    
    if (msg.indexOf('\n') < 0) {
      msg += "\n\tON THE WAY TO ";
      if (line >= 0) msg += line;
      else msg += "SOMEWHERE ELSE";
      msg += "\n\tCORRECT SOURCE AND RESUBMIT";
    }
    return msg;
  }
  
  protected static String getErrorMessage(int error) {
    switch (error) {
      case  17: return "DO YOU EXPECT ME TO FIGURE THIS OUT?";
      case  79: return "PROGRAMMER IS INSUFFICIENTLY POLITE";
      case  99: return "PROGRAMMER IS OVERLY POLITE";
      case 123: return "PROGRAM HAS DISAPPEARED INTO THE BLACK LAGOON";
      case 129: return "PROGRAM HAS GOTTEN LOST\n\tON THE WAY TO WHO KNOWS WHERE";
      case 139: return "I WASN'T PLANNING TO GO THERE ANYWAY";
      case 182: return "YOU MUST LIKE THIS LABEL A LOT!";
      case 197: return "SO!  65535 LABELS AREN'T ENOUGH FOR YOU?";
      case 200: return "NOTHING VENTURED, NOTHING GAINED";
      case 240: return "ERROR HANDLER PRINTED SNIDE REMARK";
      case 241: return "VARIABLES MAY NOT BE STORED IN WEST HYPERSPACE";
      case 275: return "DON'T BYTE OFF MORE THAN YOU CAN CHEW";
      case 436: return "THROW STICK BEFORE RETRIEVING!";
      case 533: return "YOU WANT MAYBE WE SHOULD IMPLEMENT 64-BIT VARIABLES?";
      case 562: return "I DO NOT COMPUTE";
      case 579: return "WHAT BASE AND/OR LANGUAGE INCLUDES THAT NUMBER?";
      case 621: return "ERROR TYPE 621 ENCOUNTERED";
      case 632: return "THE NEXT STACK RUPTURES.  ALL DIE.  OH, THE EMBARRASSMENT!";
      case 633: return "PROGRAM FELL OFF THE EDGE\n\tON THE WAY TO THE NEW WORLD\n\tCORRECT SOURCE AND RESUBMIT";
      case 774: return "RANDOM COMPILER BUG\n\tON MY WAY HOME\n\tTAKE YOUR TIME AND RESUBMIT";

      case 111: return "COMMUNIST PLOT DETECTED, COMPILER IS SUICIDING";
      case 127: return "SAYING 'ABRACADABRA' WITHOUT A MAGIC WAND WON'T DO YOU ANY GOOD\n\tON THE WAY TO THE CLOSET\n\tGET CHANGE AND RESUBMIT";
      case 222: return "BUMMER, DUDE!";
      case 333: return "YOU CAN'T HAVE EVERYTHING, WHERE WOULD YOU PUT IT?";
      case 444: return "IT CAME FROM BEYOND SPACE";
      case 555: return "FLOW DIAGRAM IS EXCESSIVELY CONNECTED";
      case 666: return "COMPILER HAS INDIGESTION";
      case 777: return "A SOURCE IS A SOURCE, OF COURSE, OF COURSE\n\tON MY WAY HOME\n\tGET CHANGE AND RESUBMIT";
      case 888: return "I HAVE NO FILE AND I MUST SCREAM\n\tON MY WAY HOME\n\tGET CHANGE AND RESUBMIT";
      case 999: return "NO SKELETON IN MY CLOSET, WOE IS ME!";
      case 998: return "EXCUSE ME,\n\tYOU MUST HAVE ME CONFUSED WITH SOME OTHER COMPILER\n\tON THE RUN\n\tGET A LIFE AND RESUBMIT";
      case 997: return "ILLEGAL POSSESSION OF A CONTROLLED UNARY OPERATOR.";      

      case 303: return "TOO MUCH ABSTENTION WILL DO YOU NO GOOD";
      case 601: return "WHERE DID YOU LEARN TO TYPE?\n\tON THE COMMAND LINE\n\tUPDATE USER AND RESUBMIT";
      case 701: return "INTOLERABLE MISCONFIGURATION\n\tIN THE FABRIC OF SPACE-TIME\n\tADJUST SENSORS AND RESUBMIT";
      
      default:  return "UNEXPLAINED COMPILER BUG"; // actually error 778, but seemed to be a good default
    }
  }
}
